/* _______________________________________________________________________
	TrinityCommon.h - Commonly used constants and datatypes for the
					  Trinity Architecture
					  
					  The data in this file is used to develop both
					  applications and Trinity compliant drivers
	
	first created: 7/29/2000
	
	copyright  1999-2001 by David DeHaven - all rights reserved.
 _______________________________________________________________________*/


/* currently for compiling under Darwin/MacOS X */


#ifndef _TrinityCommon_
#define _TrinityCommon_

#ifdef __cplusplus
extern "C" {
#endif

	/* abstracted for future portability */
typedef unsigned long long TRTimerType;		/* datatype returned by the timer functions and used in time stamps */

/*______________________________________________________________________
	Hardware Bus Type
  ______________________________________________________________________*/

enum {
	kTRBusTypeSerial = 'Serl',	/* legacy serial port (for SMPTE timecode generators, etc...) */
	kTRBusTypeUSB = 'USB1',
	kTRBusTypeUSB2 = 'USB2',	/* USB 2.0 */
	kTRBusTypeFireWire = 'Fire',
	kTRBudTypeFireWire_HAVI = 'HAVI',	/* FireWire with HAVI compat. firmware (different than normal FireWire) */
	kTRBusTypePCI = 'PCI ',
	kTRBusTypeCardBus = 'CBUS',
	kTRBusTypeEthernet = 'ENet',
	kTRBusTypeInternal = 'MemB'			/* internal, directly addressable by CPU */
};

/*______________________________________________________________________
	Device flags
		- Bits used in the deviceFlags field in TRDeviceInfo structure
  ______________________________________________________________________*/

enum {
	kTRDeviceIsHotPluggableBit = 0,			/* device could go away unexpectedly */
	kTRDeviceRecordingDisabledBit = 1,		/* if set, then you can't record from this device */
	kTRDeviceSlowPreviewWhileRecordBit = 2,	/* drop the framerate while recording */
	
	kTRDeviceIsHotPluggable = (1 << kTRDeviceIsHotPluggableBit),
	kTRDeviceRecordingDisabled = (1 << kTRDeviceRecordingDisabledBit),
	kTRDeviceSlowPreviewWhileRecord = (1 << kTRDeviceSlowPreviewWhileRecordBit)
};


/*______________________________________________________________________
	IO flag constants
		- Bits used in the pipeFlags field in TRPipeInfo structure
  ______________________________________________________________________*/
enum {
	kTRIOFlag_CaptureBit = 0,		/* the pipe will bring data into the host */
	kTRIOFlag_OutputBit,			/* the host can send data through the pipe */
	
	kTRIOFlag_Capture = (1 << kTRIOFlag_CaptureBit),
	kTRIOFlag_Output = (1 << kTRIOFlag_OutputBit)
};

/*______________________________________________________________________
	Error codes
		- Non-zero always means a failure
  ______________________________________________________________________*/

enum {
		/* global error codes */
	kTRGenericErr = 1,
	kTRNoMemoryErr,
	
	kTRServerDiedErr,					/* the server crash or was killed */
	
	kTRNoDevicesErr,
	kTRInvalidDeviceErr,				/* device reference specified does not exist */
	kTRDeviceNotRespondingErr,			/* something wrong with the hardware communication path */
	kTRDeviceWentAwayErr,				/* device was disconnected, you should close associated connections to free up memory */
	
	kTRInvalidConfigurationErr,			/* configuration ID specified does not exist */
	kTRConfigurationConflictErr,		/* a configuration is already active for the associated feature */
	
	kTRInvalidPipeErr,					/* the pipe ID or reference given is not valid */
	kTRPipeAlreadyPreparedErr,			/* you need to call TRPipeRelease to release the pipe first */
	kTRPipeNotPreparedErr,				/* TRPreparePipe has not been called, or failed */
	kTRPipeBusyErr,						/* pipe is in use by another client, or you called TRReadPipe after calling TRStartPipe */
	kTRPipeNotAvailableErr,				/* a conflicting pipe is currently in use */
	kTRPipeOperationNotAllowedErr,		/* You tried to do something the pipe doesn't allow, like write to a read only pipe */
	
	kTRUnsupportedSettingSelectorErr,
	kTRUnsupportedFeatureErr,
	kTRUnsupportedFormatErr,
	kTRUnsupportedModeErr
};


/*______________________________________________________________________
	Misc. constants
  ______________________________________________________________________*/
enum {
	/* any format of data is acceptable (also use for types with only one or
	   a proprietary format, like EPG or force feedback) */
	kTRFormatDontCare = -1
};

/*______________________________________________________________________
	Trinity result type
  ______________________________________________________________________*/
typedef long TrinityErr;


/*______________________________________________________________________
	Opaque device reference
		- Unique for each device and guaranteed not to change, even when
		  other devices are removed or added
		- Upon insertion, each device is assigned a new unique ID, so
		  the ID will not be the same every time it's added to the
		  device list
  ______________________________________________________________________*/
typedef unsigned long TRDeviceRef;


/*______________________________________________________________________
	Opaque pipe reference
		- Use this to communicate with the pipe (data or control) once opened
		- These are unique to each connection and have no correlation to the
		  pipe ID
  ______________________________________________________________________*/
typedef unsigned long TRPipeRef;


/*______________________________________________________________________
	Pipe information
		- Pipe ID must be unique to each device, not just a specific
		  feature or configuration, this allows connections to specific
		  pipes without any data flow until a configuration is enabled
  ______________________________________________________________________*/
struct TRPipeInfo {
	unsigned long pipeID;		/* pipe ID - unique ID for this pipe */
	char name[64];				/* descriptive name */
	char pipeDoesIO;			/* true if this pipe is used for data capture or output, false if it is only used to control a feature (eg: tuner) */
	unsigned long pipeFlags;	/* information about how I/O works on this pipe */
	unsigned long pipeType;		/* four character value describing what this pipe is for */
};
typedef struct TRPipeInfo TRPipeInfo, *TRPipeInfoPtr;


/*______________________________________________________________________
	Device Configuration information
		- Contains information about a specific configuration for a
		  device feature
  ______________________________________________________________________*/
struct TRConfigurationInfo {
	unsigned long configID;		/* unique ID for this configuration */
	char name[64];				/* descriptive name */
	
	unsigned long pipeCount;	/* number of pipes in this config */
	unsigned long pipes[1];		/* array of offsets to the pipe info records */
};
typedef struct TRConfigurationInfo TRConfigurationInfo, *TRConfigurationInfoPtr;


/*______________________________________________________________________
	Device feature information
		- Contains information about a specific feature of a device
		- Each feature must be independently controllable, though features
		  may have an indirect affect on each other, such as a remote
		  sensor for an A/V capture device
  ______________________________________________________________________*/
struct TRFeatureInfo {
	unsigned long featureID;			/* unique ID for this device feature */
	char name[64];						/* descriptive name */
	
	unsigned long configurationCount;	/* number of configurations available */
	unsigned long configurations[1];	/* array of offsets to the configuration info records */
};
typedef struct TRFeatureInfo TRFeatureInfo, *TRFeatureInfoPtr;


/*______________________________________________________________________
	Device Information
		- Accessible via TRGetDeviceInfo
		- Contains all information required by the client, including
		  device Features, configurations and pipes
  ______________________________________________________________________*/
struct TRDeviceInfo {
	char name[64];				/* descriptive name */
	
	unsigned long version;		/* hardware revision level of the product */
	char serialNumber[64];		/* unique serial number for this device (or GUID...), if supported, set to "0" if not */
	unsigned long busType;		/* physical connection type - See bus type constants */
	unsigned long deviceFlags;	/* see flags defined above */
	
	unsigned long featureCount;	/* how many features this device has */
	unsigned long features[1];	/* array of offsets to the device feature records */
};
typedef struct TRDeviceInfo TRDeviceInfo, *TRDeviceInfoPtr;


/*______________________________________________________________________
	Capture/Output buffer and buffer list stuff
		- You MUST use the allocation routines provided to allocate the
		  buffers and lists, otherwise they will not work
  ______________________________________________________________________*/

//enum {
		/*
			Set this bit when the buffer is ready to be accessed by the driver
			whether for reading or writing. When the driver is finished with a
			buffer, it will clear this bit and fire the finish proc. 
		 */
//	kTRBufferStatusBits_BufferReady = 1
//};

struct TRBuffer {
	unsigned long status;
	
	unsigned long dataType;		/* what kind of data is stored in the buffer */
	
	TRTimerType timeStamp;		/* contains the time stamp for the frame contained in this buffer (if time based) */
	unsigned long timeScale;	/* number of timeStamp units per second */
	
	unsigned long bufferSize;	/* actual size of the buffer (never change this!) */
	unsigned long dataSize;		/* how much data is stored in the buffer */
	
	void *bufferAddress;
};
typedef struct TRBuffer TRBuffer, *TRBufferPtr;

struct TRBufferList {
	long bufferCount;
	TRBufferPtr *bufferList;	/* (allocated) array of buffer pointers */
};
typedef struct TRBufferList TRBufferList, *TRBufferListPtr;

/*______________________________________________________________________
	Pipe notification messages
  ______________________________________________________________________*/

enum {
	kTRPipeNotify_DeviceSleep = 'Zzzz',		/* the device has gone to sleep/low power mode, don't expect any data from it */
	kTRPipeNotify_DeviceWake = 'Wake',		/* settings should not need to be restored */
	kTRPipeNotify_Stall = 'Hork',			/* the device has stopped working for some reason... may need to be reset */
	kTRPipeNotify_SettingChanged = 'CSet'	/* a setting has changed on the pipe being referenced, data = setting selector that changed */
};

/*______________________________________________________________________
	Trinity system notification messages
  ______________________________________________________________________*/

enum {
	kTRTrinityNotify_DeviceAdded = 'nDev',		/* a new device has been added to the list */
	kTRTrinityNotify_DeviceRemoved = 'kDev',	/* the specified device has been removed */
		// next 2 not implemented yet
//	kTRTrinityNotify_ServerSleep = 'Zzzz',		/* the server task has been suspended */
//	kTRTrinityNotify_ServerWake = 'Wake',		/* ...resumed... */
	kTRTrinityNotify_ServerDied = 'Dead'		/* the server has gone down for some reason */
};

/*______________________________________________________________________
	Capture/Output completion procs
  ______________________________________________________________________*/

/*______________________________________________________________________
	I/O finished proc
		- Called when an operation has completed
		- Buffer points to the buffer where data was stored (read) or
		  read from (write)
  ______________________________________________________________________*/
typedef void (*TRFinishProc)(void *userData, TRPipeRef pipeRef, TRBufferPtr buffer);

/*______________________________________________________________________
	Pipe Notification proc
		- Used by the Trinity Driver to communicate with the client when
		  pipe related events occur, such as the device/pipe stalling or
		  settings changed
  ______________________________________________________________________*/
typedef void (*TRPipeNotifyProc)(void *userData, TRPipeRef pipeRef, unsigned long message, unsigned long data);

/*______________________________________________________________________
	Main Trinity notification proc
		- Called for server related reasons:
			- Device added, data = device reference for the new device
			- Device removed, data = device reference for the device removed
			- Server died, data = unused
  ______________________________________________________________________*/
typedef void (*TRTrinityNotifyProc)(unsigned long message, unsigned long data);


#ifdef __cplusplus
}
#endif

/* this comment doesn't really say anything useful */

#endif /* _TrinityCommon_ */

/*______________________________________________________________________*/
